<?php

namespace Drupal\eca_config\Plugin\Action;

use Drupal\Core\Form\FormStateInterface;
use Drupal\eca\Service\Conditions;
use Symfony\Component\Yaml\Exception\ParseException;
use Symfony\Component\Yaml\Parser;

/**
 * Action to write a configuration value.
 *
 * @Action(
 *   id = "eca_config_write",
 *   label = @Translation("Config: write"),
 *   description = @Translation("Write / set a single configuration value.")
 * )
 */
class ConfigWrite extends ConfigActionBase {

  /**
   * {@inheritdoc}
   */
  public function execute(): void {
    $token = $this->tokenServices;
    $config_name = $token->replace($this->configuration['config_name']);
    $config_key = $token->replace($this->configuration['config_key']);
    $config_value = (string) $token->replaceClear($this->configuration['config_value']);
    if (strpos($config_value, ':')) {
      try {
        $parser = new Parser();
        $config_value = $parser->parse($config_value);
      }
      catch (ParseException $e) {
        \Drupal::logger('eca')->error('Tried parsing a config value in action "eca_config_write" as YAML format, but parsing failed.');
      }
    }
    $config_factory = $this->getConfigFactory();

    $config = $config_factory->getEditable($config_name);
    $current_value = $config->get($config_key);

    // Change the configuration value only when the new config value differs.
    if ($current_value !== $config_value) {
      $config->set($config_key, $config_value);
      if ($this->configuration['save_config'] === Conditions::OPTION_YES) {
        $config->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'config_value' => '',
      'save_config' => Conditions::OPTION_NO,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['config_value'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Config value'),
      '#description' => $this->t('The value to set. Supports tokens. Nested data can be set using YAML format, for example <em>front: /node</em>.'),
      '#default_value' => $this->configuration['config_value'],
      '#weight' => -7,
    ];
    $form['save_config'] = [
      '#type' => 'select',
      '#title' => $this->t('Save configuration'),
      '#default_value' => $this->configuration['save_config'],
      '#options' => [
        Conditions::OPTION_YES => $this->t('Yes'),
        Conditions::OPTION_NO => $this->t('No'),
      ],
      '#required' => TRUE,
      '#weight' => -6,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['config_value'] = $form_state->getValue('config_value');
    $this->configuration['save_config'] = $form_state->getValue('save_config');
    parent::submitConfigurationForm($form, $form_state);
  }

}
