<?php

namespace Drupal\eca\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\eca\Service\Modellers;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Import a model from a previous export.
 */
class Import extends FormBase {

  /**
   * ECA modeller service.
   *
   * @var \Drupal\eca\Service\Modellers
   */
  protected Modellers $modellerService;

  /**
   * Symfony request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected Request $request;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): Import {
    $form = parent::create($container);
    $form->modellerService = $container->get('eca.service.modeller');
    $form->request = $container->get('request_stack')->getCurrentRequest();
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'eca_import';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['model'] = [
      '#type' => 'file',
      '#title' => $this->t('File containing the exported model'),
    ];
    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Import'),
      '#button_type' => 'primary',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    parent::validateForm($form, $form_state);
    $all_files = $this->request->files->get('files', []);
    if (empty($all_files)) {
      $form_state->setErrorByName('model', 'No file provided.');
      return;
    }
    /** @var \Symfony\Component\HttpFoundation\File\UploadedFile $file */
    $file = reset($all_files);
    if ($file === NULL) {
      $form_state->setErrorByName('model', 'No file provided.');
      return;
    }
    $filename = $file->getRealPath();
    if (!file_exists($filename)) {
      $form_state->setErrorByName('model', 'Something went wrong during upload.');
      return;
    }
    [$name] = explode('.', $file->getClientOriginalName());
    [$modellerId, $id] = explode('-', $name);
    $modeller = $this->modellerService->getModeller($modellerId);
    if (!$modeller) {
      $form_state->setErrorByName('model', 'The required modeller is not available.');
      return;
    }

    // Verify, that ID is not used yet.
    $data = file_get_contents($filename);
    $modeller->setModeldata($data);
    if ($eca = $this->modellerService->loadModel($modeller->getId())) {
      $form_state->setErrorByName('model', 'Model with that ID already available with the name "' . $eca->label() . '". Delete that first, when you really want to import this file.');
    }

    // Test import the model and display missing dependencies if it fails.
    $eca = $modeller->createNewModel($modeller->getId(), $data);
    $modeller->readComponents($eca);
    if ($modeller->hasError()) {
      // Reading components didn't succeed and recognized errors got pushed to
      // Drupal's messenger.
      return;
    }
    try {
      $eca->calculateDependencies();
    }
    catch (\LogicException $e) {
      $form_state->setErrorByName('model', $e->getMessage());
      return;
    }

    // Prepare for final import in submit handler.
    $form_state->setValue('model', [
      'filename' => $filename,
      'modellerId' => $modellerId,
      'id' => $id,
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $model = $form_state->getValue('model');
    if ($modeller = $this->modellerService->getModeller($model['modellerId'])) {
      $modeller->save(file_get_contents($model['filename']), NULL, FALSE);
    }
    $form_state->setRedirect('entity.eca.collection');
  }

}
